## Please ensure that
# R and RStudio are updated
# All data files are under the same working directory as this script
# All required packages are installed
# Any enquiries can be directed towards lichen.ma@psychology.su.se

## Loading all necessary packages
# ------------------------------------------------------------------------------
library(XML)
library(plyr)
library(dplyr)
library(DT)
library(readxl)
library(tableone)
library(car)
library(knitr)
library(kableExtra)
library(ggplot2)
library(gridExtra)
library(psych)
library(htmlTable)
library(reshape2)
library(nlme)
library(ez)

## define function to export plots as tiffs with specified dimension, 
# resolution and background color:
plotExportTiff <- function(filename = NA, plot = NA, dpi = 150, width = 650, 
                           height = 350, background = "white"){
  tiff(filename = filename, width= (width *(dpi/72)), 
       height= (height * (dpi/72)), res = dpi, bg = background)
  print(p)
  dev.off()
}
# ------------------------------------------------------------------------------
  
## Create function that parses all xml data files in a folder
# ------------------------------------------------------------------------------
parse_xml <-function(FileName) {
 xdf <- xmlParse(FileName)

 # get identifying info from FileName
 ppid <- sub('(.*\\/)(.{8})(.*)', "\\2", FileName) 
 # 8 characters after the final /
 group <- sub('(.*\\/)(G\\d)(.*)', "\\2", FileName) 
 # G followed by a digit preceded by a /
 time <- sub("(.*_)(.*)(--.*)", "\\2", FileName) 
 # text inbetween a _ and --
 
 # to get the trial numbers, read the XML file nodeset and subtract 
 # the trialnums from these printed 'as character'. 
 nodes <- getNodeSet(xdf,"//Trial")
 trialNum <- rep(NA, length(nodes))
 for (n in 1: length(nodes)) {
 trialNum[n] <-  as.numeric(sub("\\D*(\\d+).*", "\\1", 
                                     as(nodes[[n]], "character")))
 }
    
 # finally, column bind the identifying info, trialNum, and the actual data 
 # together with a call to 'xmlToDataFrame' for each node. 
 xdf <- cbind(ppid, group, time, trialNum, 
                 xmlToDataFrame(nodes=getNodeSet(xdf,"//Trial"))[c(
                   "timeStamp",
                   "stimulusProbed",
                   "topStimulus", 
                   "botStimulus",
                   "probeType", 
                   "probePlacement",
                   "reactionTime",
                   "wasCorrectResponse")] )
  }
# ------------------------------------------------------------------------------

## Importing attentional bias (reaction time) data
# ------------------------------------------------------------------------------
# the parse_xml function will be run in a loop so that all 4 subfolders 
# are processed in one go:
todolist <- c("G1", "G2", "G3", "G4")

trialdf = data.frame(NULL)

for(todo in 1:length(todolist)) {
# first search the folder and all its subfolders for all files that have the 
# required parts in it. I.e. for the first item on to do list it searches all 
# files with 'G1', followed by 'pre' anywhere in the filename: 
  tofind <- sub("_", ".*_", todolist[todo]) 
# the _ is added so that files with _pre are taken
  files <- list.files(".", 
                      recursive = T) [grepl
                                      (tofind, list.files
                                        (".", recursive = T))]
  
# Create the required data frames as NULL
  trialsetdf = data.frame(NULL)
  
# initiate a for loop that applies the same code to every file
  for(i in 1:length(files)) {
    
    trialsetdf <- ldply(files[i],parse_xml) 
    # apply parse_xml function defined above
    
    # convert reactionTime column to numeric
    trialsetdf$reactionTime = as.numeric(as.character(trialsetdf$reactionTime))
    
    # derive the trial type from the stimulus names 
    # (_D and _N in the stimulus filenames indicate Disgust & Neutral)
    trialsetdf$trialstimtype [grepl("_N", trialsetdf$topStimulus) 
                              & grepl("_N", trialsetdf$botStimulus)] <- "NN"
    trialsetdf$trialstimtype [grepl("_N", trialsetdf$topStimulus) 
                              & grepl("_D", trialsetdf$botStimulus)] <- "DN"
    trialsetdf$trialstimtype [grepl("_D", trialsetdf$topStimulus) 
                              & grepl("_N", trialsetdf$botStimulus)] <- "DN"
    
    # derive the congruency for the DN trials: 
    trialsetdf$congruency [trialsetdf$trialstimtype == "DN" 
                           & trialsetdf$stimulusProbed == "emotional" ] <- 1
    trialsetdf$congruency [trialsetdf$trialstimtype == "DN" 
                           & trialsetdf$stimulusProbed == "neutral" ] <- 0
    trialsetdf$congruency [trialsetdf$trialstimtype == "NN" ] <- 99
    
    # multiply the variable reactiontime so that it shows 
    # RT in miliseconds (and also round it)
    trialsetdf$reactionTime <- round(1000 * trialsetdf$reactionTime)
    
    # rowbind this trialset into the dataframe trialdf:
    trialdf <- rbind(trialdf, trialsetdf)
  } 
}

table(trialdf$ppid) # number of trials per participant
# Four participants had 199 trials instead of 200. 
# These are the first participants in each of the four groups and it is their 
# first trials that are missing. We suspect that there were some coding error 
# caused the failure to log the first trial for the first participant 
# in each group.

table(trialdf$trialNum) 
# number of trials for each value trialNum - most are 100 represting 
# two times (pre/post) 100 trials for 100 pp; except for trialNum == 1, 
# where there are four trials missing (for each of the four conditions, 
# the first trial of the first pp in each condition)
# ------------------------------------------------------------------------------

## Attentional bias data cleaning 
# ------------------------------------------------------------------------------
# removing all neutral-neutral trials:
trialdf <- trialdf[trialdf$trialstimtype == "DN", ]

# define filters to exclude trials above 2000ms and below 150ms:
rtMax <- 2000
rtMin <- 150
SDbound <- 2

# if less than 80% of congruent or incongruent trials are valid at either 
# pre or post assessment, the participant will be marked for exclusion
minPrcValid <- 80 

# filter trials: 
sdFilter <- NULL
sdFilter <- do.call(data.frame,
                    aggregate(reactionTime ~ ppid + congruency + time, 
                              trialdf [trialdf$reactionTime >= rtMin 
                                       & trialdf$reactionTime <= rtMax,], 
                              function(x) c(mean = mean(x), sd = sd(x))))

# the above code creates a dataframe (called sdFilter), 
# that contains for each participant their mean and sd RT separate for each 
# combination of congruency and time (pre/post). 
# In the code below we use these to compute the mean +/- 2sd boundaries 
# for each participant.

# compute mean +/- 2sd boundaries in the sdFilter dataframe:
sdFilter$upperSDbound <- sdFilter$reactionTime.mean + SDbound * 
  sdFilter$reactionTime.sd
sdFilter$lowerSDbound <- sdFilter$reactionTime.mean - SDbound * 
  sdFilter$reactionTime.sd

# names(sdFilter) [names(sdFilter) %in% names(trialdf)] 
# the two data frames share the variables Subject, SOA, Condition and Congruency. 
# therefore a call to merge will match the two df on the shared vars.

trialdf <- merge(trialdf, sdFilter)

names(trialdf) 
# the variables "reactionTime.mean", "reactionTime.sd", "upper" & "lower" 
# are now merged into trialdf - and can be used to filter the trials. 

# create a filter variable with initial status "keep" for all trials
trialdf$trialfilter <- "keep"

# mark trials with response times < rtMin and > rtMax
trialdf$trialfilter [trialdf$reactionTime > rtMax] <- paste(">", rtMax, "ms")   
trialdf$trialfilter [trialdf$reactionTime < rtMin] <- paste("<", rtMin, "ms")

# mark trials with response times above or below the sdboundary defined 
# in sdfilter that are not < rtMin or > rtMax:
trialdf$trialfilter [trialdf$reactionTime > trialdf$upperSDbound 
                     & trialdf$trialfilter == "keep"] <- paste("above", SDbound,
                                                               "sd")
trialdf$trialfilter [trialdf$reactionTime < trialdf$lowerSDbound 
                     & trialdf$trialfilter == "keep"] <- paste("below", SDbound,
                                                               "sd")

# mark trials with response times above or below the sdboundary as defined 
# in sdfilter that are also < rtMin or > rtMax:
trialdf$trialfilter [trialdf$reactionTime > trialdf$upperSDbound 
                     & trialdf$trialfilter == "> 2000 ms"] <- paste ("above", 
                                                                     SDbound, 
                                                                     "sd & >", 
                                                                     rtMax, "ms")

trialdf$trialfilter [trialdf$reactionTime < trialdf$lowerSDbound 
                     & trialdf$trialfilter == "< 150 ms"]  <- paste ("below", 
                                                                     SDbound, 
                                                                     "sd & <", 
                                                                     rtMin, "ms")

# mark error trials:
trialdf$trialfilter [trialdf$wasCorrectResponse == "false"] <- "error"

# now first flag up the participants who failed to reach the minimum percentage 
# of valid trials: 
keepcounter <- aggregate(trialfilter ~ ppid + time + congruency , 
                         trialdf[trialdf$trialfilter== "keep",], 
                         function(x) c("percentValid" = length(x)/40*100)) 
# count remaining number of valid trials for each ppid at both timepoints 
# for incongruent & congruent trials 

names(keepcounter) <- c("ppid", "time", "congruency", "prc_validTrials")
# rename vars

datatable(keepcounter)
# for most individual subdatasets we have nearly 100 % of the trials remaning or 
# otherwise a small number of trials removed - yet there are 5 pp for which 
# < 80% of the trials are avaible for either one or both the subsets of 
# congruent/incongruent trials at pre or post. 

perc_Filter <- unique(keepcounter
                      [keepcounter$prc_validTrials < minPrcValid,]$ppid)

# create two datasets: one for all pp, 
# the other without the pp for which we'd retain < minPrcValid trials. 

trialdfAnalyse <- trialdf[! trialdf$ppid %in% perc_Filter,]

# overview of all trials nominated for removal and the reasons for each, 
# per subject and time for the analysis set:
table(trialdfAnalyse$trialfilter, trialdfAnalyse$time)

# in percentages of trials removed:
(table(trialdfAnalyse$trialfilter, trialdfAnalyse$time) / 
    (80 * length(unique(trialdfAnalyse$ppid))))*100 
# 80 DN trials *length(unique(trialdfAnalyse$ppid) 
# which is the number of ppid retained after filtering

# filter trialdfAnalyse into a dataframe called tdf, 
# keeping only the trials labeled keep: 
tdf <- trialdfAnalyse[trialdfAnalyse$trialfilter == "keep",]
# ------------------------------------------------------------------------------

## Calculate attentional bias
# ------------------------------------------------------------------------------
# aggregate trialdata to get the probe.RT mean SD and number for 
# each combination of ppid, congruency, and timepoint:
BIdf <- do.call(data.frame,
                aggregate(reactionTime ~ ppid + congruency + time + group, tdf, 
                          function(x) c(mean = mean(x))))

# reshape agg df to one line for eahc combination of subject and trialtype, 
# with the congruency variable as the timevar to get separate varabiles for 
# congruent (1) incongruent (0) and the neutneut/nocongruency (2) data
BIdf <- reshape(BIdf, idvar = c("ppid", "time", "group"), 
                timevar = "congruency", direction = "wide")

names(BIdf) <- c("ppid", "time", "group", "rt.Incongruent", "rt.Congruent")

# compute BI as probe.RT.mean for incongruent (probe.RT.mean.0) minus the 
# probe.RT.mean for congruent trials (probe.RT.mean.1).
BIdf$BI <- BIdf$rt.Incongruent - BIdf$rt.Congruent

# reshape to wide again. One row per subject, 
# and the variables receive a time indicator. 
BIdf <- reshape(BIdf, idvar = c("ppid", "group"), timevar = "time", 
                direction = "wide")
# ------------------------------------------------------------------------------

## Import questionnaire data 
# ------------------------------------------------------------------------------
sdf <- read_excel("Data_files/outcome_measures_and_demographic_data.xlsx", 1)
# Note: participant 1398mwss was incorrectly input as 1398wmss into the VR data. 
# The username was manually changed in the behavioural data file as 
# the bias data were hard-coded

# replace any values [missing data] with NA
sdf[ sdf == "[missing data]" ] <- NA

# several questionnaire variables are in chr class - blanket convert to numeric: 
sdf[,grepl(paste("LSAS", "PHQ", "GAD", "DERS", "BBQ", sep = "|"), 
           names(sdf))]<- lapply(sdf[,grepl(paste("LSAS", "PHQ", "GAD", "DERS", 
                                                  "BBQ", sep = "|"), 
                     names(sdf))] , function(x) as.numeric(as.character(x)))

# simplify the colnames for Gender and Education 
# which initially hold coding info too:
colnames(sdf)[grepl("Gender", colnames(sdf))] <- 'Gender'
# 0 = female, 1 = male
colnames(sdf)[grepl("Education", colnames(sdf))] <- 'Education'
# 0 = secondary and below, 1 = tertiary and beyond

# remove 5 excluded participants 
# (1063fkzk, 1118mmvn, 1298fbpr, 1682crkh, 1707dpbh)
sdf <- sdf[! sdf$ppid  %in% 
             c("1063fkzk", "1118mmvn", "1298fbpr", "1682crkh", "1707dpbh"), ]

# Merge BIdf and sdf to include all relevant variables for analysis:
mergedf <- merge (BIdf, sdf[, c('ppid', 'LSAS.pre', 'LSAS.post', 'PHQ.pre', 
                                'GAD.pre', 'DERS.pre', 'BBQ.pre', 'Age', 
                                'Gender', 'Education')])
# ------------------------------------------------------------------------------

## Compare group differences at baseline
# ------------------------------------------------------------------------------
# take pre bias, questionnaire and demographic variables from mergedf into t1df:
t1df <- mergedf[ ,names(mergedf) %in% 
                   c("ppid", "group", "Gender", "Education", "Age", "LSAS.pre", 
                     "PHQ.pre", "GAD.pre", "DERS.pre", "BBQ.pre", "BI.pre")]

# add labels for gender and education
t1df$Gender [t1df$Gender == '0'] <- 'female'
t1df$Gender [t1df$Gender == '1'] <- 'male'
t1df$Education [t1df$Education == '0'] <- 'tecondary'
t1df$Education [t1df$Education == '1'] <- 'tertiary'

# create a tableOne object 
t1vars <- c("Gender", "Education", "Age", "BI.pre", "LSAS.pre", "PHQ.pre",
            "GAD.pre", "DERS.pre", "BBQ.pre")
t1varsF <- c("Gender", "Education")

table1 <- CreateTableOne(vars = t1vars, factorVars = t1varsF, strata = "group", 
                         data= t1df, includeNA = FALSE,
                         test = TRUE, testApprox = chisq.test, 
                         argsApprox = list(correct = TRUE),
                         testExact = fisher.test, 
                         argsExact = list(workspace = 2 * 10^5),
                         testNormal = oneway.test, 
                         argsNormal = list(var.equal = TRUE),
                         testNonNormal = kruskal.test, 
                         argsNonNormal = list(NULL), smd = TRUE)

table1 
# table 1 without styling. 
# The test column indicats whether normal or non-normal tests 
# were used to derive the p-value.

summary(table1)

# test for homogeneity of variance using 
# Levene's test on all continuous variables:
t1varsC <- c( "Age", "BI.pre", "LSAS.pre", "PHQ.pre", "GAD.pre", 
              "DERS.pre", "BBQ.pre") # vector of all continuous variables

for(v in 1: length(t1varsC)){
  test <- leveneTest(t1df[ , paste(t1varsC[v])]  ~  group, data = t1df) 
  print(t1varsC[v])
  print(test)
}
# All nonsignificant
# ------------------------------------------------------------------------------

## Assessing correlations between bias and anxiety
# ------------------------------------------------------------------------------
mergedf$BI.change <- mergedf$BI.post - mergedf$BI.pre  
# compute change for BI scores post-pre
mergedf$LSAS.change <- mergedf$LSAS.post - mergedf$LSAS.pre  
# compute change for LSAS scores post-pre

devtools::source_gist("f1e6b1531a274eb623cbc7e54727296a", 
                      filename = "correlation table simple.r") 
# source code by Anne-Wil Kruijt for drawing correlation table

corrtable(mergedf[,c("BI.pre", "BI.post", "BI.change", "LSAS.pre", 
                     "LSAS.post", "LSAS.change")], adjust = "none")

# create three subplots, that will be presented side to side with grid.arrange
cor.pre <- ggplot(mergedf, aes(x= LSAS.pre, y = BI.pre)) +
  geom_smooth(method='lm',formula=y~x, col = "skyblue3") + 
  geom_point(shape = 16, colour = "skyblue2") + 
  geom_point(shape = 1, colour = "black", stroke = .6) + 
  theme_minimal() + theme(panel.grid = element_blank(), 
                          axis.line = element_line(colour = "gray")) +
  labs(x = "LSAS-SR pre", y = "BI pre (ms) ") +
  scale_x_continuous(limits = c(5, 125))  +
  scale_y_continuous(limits = c(-160, 145))

cor.post <- ggplot(mergedf, aes(x= LSAS.post, y = BI.post)) +
  geom_smooth(method='lm',formula=y~x, col = "skyblue3") + 
  geom_point(shape = 16, colour = "skyblue2") + 
  geom_point(shape = 1, colour = "black", stroke = .6) + 
  theme_minimal() + theme(panel.grid = element_blank(), 
                          axis.line = element_line(colour = "gray")) +
  labs(x = "LSAS-SR post", y = "BI post (ms) ") +
  scale_x_continuous(limits = c(5, 125))  +
  scale_y_continuous(limits = c(-160, 145))

cor.change <- ggplot(mergedf, aes(x= LSAS.change, y = BI.change)) +
  geom_smooth(method='lm',formula=y~x, col = "skyblue3") + 
  geom_point(shape = 16, colour = "skyblue2") + 
  geom_point(shape = 1, colour = "black", stroke = .6) + 
  theme_minimal() + theme(panel.grid = element_blank(), 
                          axis.line = element_line(colour = "gray")) +
  labs(x = "LSAS-SR change", y = "BI change (ms) ")  +
  scale_x_continuous(limits = c(-65, 15))  +
  scale_y_continuous(limits = c(-160, 145))

grid.arrange(cor.pre, cor.post, cor.change, nrow=1) 

# to export as tiff with various options: remove #
# dpi = 1000
# tiff(filename = "Fig3 BI LSAS corr plots.tiff", width= (650 *(dpi/72)), height= (350 * (dpi/72)), res = dpi, bg = "white")
# grid.arrange(cor.pre, cor.post, cor.change, nrow=1)
# dev.off()
# ------------------------------------------------------------------------------

## Repeated-measures ANOVA to assess bias change 
# ------------------------------------------------------------------------------
# create long data frame for attentional bias data
BIlong <- melt(mergedf, id.vars = "ppid",
               measure.vars = c("BI.pre", "BI.post"))

# merge in the group variable from BIdf: 
BIlong <- merge(BIlong, BIdf[, c("ppid", "group")], by = "ppid")

# create variables condition (mock/active) & 
# stimuli (2D/3D) based on value of group
BIlong$condition_string [BIlong$group %in% c("G1", "G3")] <- "mock"
BIlong$condition_string [BIlong$group %in% c("G2", "G4")] <- "active"

BIlong$condition [BIlong$group %in% c("G1", "G3")] <- 0
BIlong$condition [BIlong$group %in% c("G2", "G4")] <- 1
BIlong$condition <- as.numeric(BIlong$condition)

BIlong$stimuli_string [BIlong$group %in% c("G1", "G2")] <- "2D"
BIlong$stimuli_string  [BIlong$group %in% c("G3", "G4")] <- "3D"

BIlong$stimuli [BIlong$group %in% c("G1", "G2")] <- 0
BIlong$stimuli[BIlong$group %in% c("G3", "G4")] <- 1
BIlong$stimuli <- as.numeric(BIlong$stimuli)

BIlong$time_string <- sub('.*\\.', '', BIlong$variable) 
# extract string following the last period in variable "variable" 
# to obtain the timepoint indicators (pre, post, 1wk, 3mo)
BIlong$time_id [BIlong$time_string == "pre"] <- 1
BIlong$time_id [BIlong$time_string == "post"] <- 2


BIlong$id_id <- BIlong$ppid
BIlong$BI <- as.numeric(BIlong$value)

# ANOVA for BI change
ezANOVA(data = BIlong,
        dv = BI,
        within = time_string,
        between = group,
        wid = ppid,
        type = 3)

# No significant effects
# Levene's test can be conducted by running above ANOVA 
# without the 'within =' line. Nonsignificant.
# ------------------------------------------------------------------------------

## Assessing reliability of attentional bias
# ------------------------------------------------------------------------------
# set apart the baseline measurement trials from the trialdfAnalyse 
# (which holds all trials for all included participants)
trialpredf <- trialdfAnalyse [trialdfAnalyse$time == "pre",]  

# install splithalf package from Github:
devtools::install_github("https://github.com/sdparsons/splithalf")

# the splithalf package is still in development - here we used the 
# development version from github and *NOT* the CRAN version! 
# After install the version number is 0.5.2 in rstudio's package browser 
# (and for some reason not 0.3.1 as given in the current description on github)

# reliablity for the mean RT by congruency:
require('splithalf')
splithalf(trialpredf, outcome = "RT", score = "average", halftype = "random", 
          permutations = 5000, var.RT = "reactionTime", var.participant = "ppid",
          var.trialnum = "trialNum", var.condition = "congruency", 
          conditionlist = c(0, 1))

# this is reliablity for the difference score 
# (i.e BI - focus on the spearmanbrown outcome):
BIrel <- splithalf(trialpredf, outcome = "RT", score = "difference", 
                   var.compare = "congruency", compare1 = 0, compare2 = 1, 
                   halftype = "random", permutations = 5000, 
                   var.RT = "reactionTime", var.participant = "ppid",  
                   var.trialnum = "trialNum")
BIrel

relEstBI <- BIrel$spearmanbrown 
# store the value of spearman brown as variable 'relEstBI' 
# to use in the reliable change analyses
# ------------------------------------------------------------------------------

## Calculate reliable change indices, 
## and classify outcome based on reliable change index (RCI)
# ------------------------------------------------------------------------------
# fetch JTRCI (Jacobson Truax/RCI package) by AWK from github:
devtools::install_github("AWKruijt/JTRCI", force = TRUE)

# compute RCI:
JTRCI::JTRCI(x.pre= BIdf$BI.pre, x.post = BIdf$BI.post, ppid = BIdf$ppid, 
             reliability = relEstBI, JT.crit = "auto", plot = T, type = "RCI")


# create a custom RCI plot with group indicators: 

JTRCIdf$RCIclassPlot <-  JTRCIdf$class_RCI

for (l in 1: length(levels(JTRCIdf$RCIclassPlot))) {
  levels(JTRCIdf$RCIclassPlot)[l] <- paste0(levels
                                            (JTRCIdf$RCIclassPlot)[l], ": ", 
                                            table(JTRCIdf$RCIclassPlot)[[l]])
}

# fix the colors to the classification labels:
levels(JTRCIdf$RCIclassPlot)

# created vector of plot colours linked to specific level names:  
cols <- c("coral1", "skyblue3", "orchid3" )

names(cols) <- c(levels(JTRCIdf$RCIclassPlot)
                 [grep("deteriorated", levels(JTRCIdf$RCIclassPlot))], 
                 levels(JTRCIdf$RCIclassPlot)
                 [grep("no reliable change", levels(JTRCIdf$RCIclassPlot))], 
                 levels(JTRCIdf$RCIclassPlot)
                 [grep("improved", levels(JTRCIdf$RCIclassPlot))]) 

# the fix the diagonal lines in the rigth position obtain the Sdiff from the JTRCIdf:
Sdiff <- unique(JTRCIdf$Sdiff)

# match group info into JTRCIdf:
JTRCIdf <- merge(JTRCIdf, BIdf[,names(BIdf) == c("ppid", "group") ], 
                 by = "ppid")
JTRCIdf$groupPlot [JTRCIdf$group == "G1"] <- "2D mock"
JTRCIdf$groupPlot [JTRCIdf$group == "G2"] <- "2D active"
JTRCIdf$groupPlot [JTRCIdf$group == "G3"] <- "3D mock"
JTRCIdf$groupPlot [JTRCIdf$group == "G4"] <- "3D active"

JTRCIdf$groupPlot <- factor(JTRCIdf$groupPlot, 
                            levels = c("2D mock", "2D active", 
                                       "3D mock", "3D active"))

p <- ggplot(JTRCIdf, aes(x=pre, y=post, 
                         shape = groupPlot, colour = RCIclassPlot)) + 
  scale_x_continuous(limits = c(-200, 220)) +
  scale_y_continuous(limits = c(-200, 220)) +
  geom_abline(intercept = 0, slope = 1) + 
  geom_abline(intercept =  Sdiff *  1.96 , slope = 1, linetype =3) + 
  geom_abline(intercept =  Sdiff * -1.96 , slope = 1, linetype =3) +
  annotate("text", x = 10, y = 195, size = 3, 
           label = "reliable increase \n (> 1.96 Sdiff)")  +
  annotate("rect", xmin = 85, xmax = 135, ymin =85, 
           ymax = 135, size = 3, fill = "white")  +
  annotate("text", x = 115, y =115,  size = 3, label = "no reliable change")  +
  annotate("text", x = 195, y = 10,  size = 3, 
           label = "reliable decrease \n (> 1.96 Sdiff)")  +
  geom_point( size = 2.5) + 
  theme_classic() + 
  theme(panel.grid.major = element_line(color = "gray95"))  + 
  scale_colour_manual(values = cols,drop = FALSE) + 
  scale_shape_manual(values=c(0, 15,2,17))+ 
  guides(colour = guide_legend(order = 1), shape = guide_legend(order = 2)) + 
  labs(x = "bias pre training", y = "bias post training", 
       colour = "RCI classification:", shape = "training condition:")

p

# to export as tiff with various options: remove #
# plotExportTiff(filename = "Fig4 BI RCI plot.tiff", plot= p, width = 500, height = 350, dpi = 1000)
# ------------------------------------------------------------------------------

## Create long data frame for LSAS
# ------------------------------------------------------------------------------
lsasdfLong <- melt(sdf, id.vars = "ppid",
                   measure.vars = c("LSAS.pre", "LSAS.post", 
                                    "LSAS.1wk", "LSAS.3mo"))

names(lsasdfLong) 

# merge in the group variable from BIdf: 
lsasdfLong <- merge(lsasdfLong, BIdf[, c("ppid", "group")], by = "ppid")

# create variables condition (mock/active) & stimuli (2D/3D) based on value of group
lsasdfLong$condition_string [lsasdfLong$group %in% c("G1", "G3")] <- "mock"
lsasdfLong$condition_string [lsasdfLong$group %in% c("G2", "G4")] <- "active"

lsasdfLong$condition [lsasdfLong$group %in% c("G1", "G3")] <- 0
lsasdfLong$condition [lsasdfLong$group %in% c("G2", "G4")] <- 1
lsasdfLong$condition <- as.numeric(lsasdfLong$condition)

lsasdfLong$stimuli_string [lsasdfLong$group %in% c("G1", "G2")] <- "2D"
lsasdfLong$stimuli_string  [lsasdfLong$group %in% c("G3", "G4")] <- "3D"

lsasdfLong$stimuli [lsasdfLong$group %in% c("G1", "G2")] <- 0
lsasdfLong$stimuli[lsasdfLong$group %in% c("G3", "G4")] <- 1
lsasdfLong$stimuli <- as.numeric(lsasdfLong$stimuli)

lsasdfLong$time_string <- sub('.*\\.', '', lsasdfLong$variable) 
# extract string following the last period in variable "variable" to 
# obtain the timepoint indicators (pre, post, 1wk, 3mo)
lsasdfLong$time_id [lsasdfLong$time_string == "pre"] <- 1
lsasdfLong$time_id [lsasdfLong$time_string == "post"] <- 2
lsasdfLong$time_id [lsasdfLong$time_string == "1wk"] <- 3
lsasdfLong$time_id [lsasdfLong$time_string == "3mo"] <- 4

lsasdfLong$id_id <- lsasdfLong$ppid

lsasdfLong$LSAS <- as.numeric(lsasdfLong$value)
# ------------------------------------------------------------------------------

## Model comparison using nlme()
# ------------------------------------------------------------------------------
# Use maximum likelihood for model comparison. 

# Null model with intercept only
model_null <- gls(LSAS ~ 1,
                  data = lsasdfLong,
                  method = 'ML',
                  na.action = 'na.omit')
# AIC(model_null) = 3302.8

# add random intercept of each participant to model
model_ri <- lme(LSAS ~ 1, random = ~ 1|ppid,
                data = lsasdfLong,
                method = 'ML',
                na.action = 'na.omit')
# AIC(model_ri) = 3082.1

# add random intercept and slope for each participant
model_ris <- lme(LSAS ~ 1, random = ~ time_id|ppid,
                 data = lsasdfLong,
                 method = 'ML',
                 na.action = 'na.omit')
# AIC(model_ris) = 3031.9

# add fixed effect of time only
model_time <- lme(LSAS ~ time_id, random = ~ time_id|ppid,
                  data = lsasdfLong,
                  method = 'ML',
                  na.action = 'na.omit')
# AIC(model_time) = 2979.3

# add fixed effect of condition only
model_condition <- lme(LSAS ~ condition, random = ~ time_id|ppid,
                       data = lsasdfLong,
                       method = 'ML',
                       na.action = 'na.omit')
# AIC(model_condition) = 3033.4

# fixed effect of time, condition, and interaction
model_tci <- lme(LSAS ~ time_id + condition + time_id:condition, 
                 random = ~ time_id|ppid,
                 data = lsasdfLong,
                 method = 'ML',
                 na.action = 'na.omit')
# AIC(model_tci) = 2969.3

# This model has the lowest AIC value of all models fitted
# Significant interaction - mock training condition reduces LSAS score 
# significantly more than active training condition over time.

# fixed effect of all main effects plus all 2x interactions
model_tcs2i <- lme(LSAS ~ time_id + condition + stimuli 
                   + time_id:condition
                   + time_id:stimuli
                   + condition:stimuli,
                   random = ~ time_id|ppid,
                   data = lsasdfLong,
                   method = 'ML',
                   na.action = 'na.omit')
# AIC(model_tcs2i) = 2973.3

# Full model
model_full <- lme(LSAS ~ time_id + condition + stimuli 
                  + time_id:condition
                  + time_id:stimuli
                  + condition:stimuli
                  + time_id:condition:stimuli,
                  random = ~ time_id|ppid,
                  data = lsasdfLong,
                  method = 'ML',
                  na.action = 'na.omit')
# AIC(model_full) = 2973.9

# list AIC for all models:
AIC(model_null, model_ri, model_ris, model_time, model_condition, 
    model_tci, model_tcs2i, model_full)

# Model with only time and condition has lower AIC than full model. 
# Direct comparison of two models to see if the simpler model has better fit.
anova (model_full, model_tci) 
# Model_tci no significantly better fit than the full model.

# both the tci and full models have a better fit than the null model:
anova (model_null, model_full) 
anova (model_null, model_tci) 

summary(model_full)
summary(model_tci)
# ------------------------------------------------------------------------------

## Plot showing LSAS score change over time, separated by group
# ------------------------------------------------------------------------------
LSASplotDF <- do.call(data.frame, 
                      aggregate(LSAS ~ group + time_id, lsasdfLong, 
                                function(x) c(mean = mean(x), 
                                              sd = sd(x), 
                                              n = length(x), 
                                              se = sd(x) / sqrt(length(x)))))

pd <- position_dodge(0.2)

# plot with 95% ci eror bars (1.96 * se on either side)
p <- ggplot(LSASplotDF, aes(x = as.factor(time_id), 
                            y = LSAS.mean, group = group)) + 
  theme_bw() + theme(panel.border = element_blank(), 
                     panel.grid = element_blank(), axis.line = element_line()) +
  scale_shape_manual(values = c(21, 19, 22, 15), 
                     labels=c("2D mock","2D active","3D mock", "3D active")) + 
  geom_line(position = pd)  +
  geom_errorbar(aes(ymin=LSAS.mean - 1.96 * LSAS.se, 
                    ymax=LSAS.mean + 1.96 * LSAS.se), width=0.2, position = pd) + 
  geom_point(aes(shape = group), size = 4,fill = "white", position = pd) +
  labs(x = "\ntime", y = "LSAS-SR\n") +
  scale_y_continuous(limits = c(0, 80)) +
  scale_x_discrete(labels = c("pre", "post", "1 week", "3 months"))

p 

# to export as tiff with various options: remove #
# plotExportTiff(filename = "Fig5 LSAS by group over time.tiff", 
# plot= p, width = 500, height = 350, dpi = 1000)
# ------------------------------------------------------------------------------

## Jacobson and Truax clinical significant change analysis on LSAS
# ------------------------------------------------------------------------------
# first load the LSAS individual data at pre:
itemdf <- read_excel("Data_files/lsas_pre_individual_items.xlsx", 1)

# remove 5 excluded participants 
# (1063fkzk, 1118mmvn, 1298fbpr, 1682crkh, 1707dpbh)
itemdf <- itemdf[! itemdf$ppid  %in% c("1063fkzk", "1118mmvn", "1298fbpr", 
                                       "1682crkh", "1707dpbh"), ]

relEstLSAS <- psych::alpha(itemdf[,4:length(itemdf)])$total$std.alpha  
# use psych packages alpha function to obtain reliabiliyt estimate for the data 
# which are stored in itemdf collumns 5:end of the dataframe. Store the value of 
# standardized alpha as 'relEstLSAS'. 

# to obtain the Jacobson-Truax indices, the JT_RCI script is used on the 
# aggregated symptom data, which is stored in sdf. 

# pre to post training: 
JTRCI::JTRCI(x.pre= sdf$LSAS.pre, x.post = sdf$LSAS.post, reliability = relEstLSAS, ppid = sdf$ppid)
# pre to 1 wk follow-up:
JTRCI::JTRCI(x.pre= sdf$LSAS.pre, x.post = sdf$LSAS.1wk, reliability = relEstLSAS, ppid = sdf$ppid)
# pre to 3 mo follow-up:
JTRCI::JTRCI(x.pre= sdf$LSAS.pre, x.post = sdf$LSAS.3mo, reliability = relEstLSAS, ppid = sdf$ppid)
# ------------------------------------------------------------------------------

## Plot individuual participant's LSAS score change across time
# ------------------------------------------------------------------------------
# create data frame with change data for the three post measurements in long format.  
pdf <- NULL # initiate df
pdf$ppid <- JTRCIdf$ppid # start plotdf with JTRCIdfs ppid 
pdf <- as.data.frame(pdf)
pdf$time <- "BL" # add time var with value BL
pdf$change <- 0 # add change var with value 0 for all at BL
pdf$RCI <- 0 # add RCI var with value 0 for all at BL
pdf$JTclass_RCI <- NA # add RCI var with value NA for all at BL

# run the JT_RCC function on pre-post data to get the first set of follow up data data (the post data, outputed in JTRCIdf)
JTRCI::JTRCI(x.pre= sdf$LSAS.pre, x.post = sdf$LSAS.post, reliability = relEstLSAS, ppid = sdf$ppid, plot = F, table = F)
JTRCIdf$time <- "post" # add time var with value post to the JTRCIdf dataframe

pdf <- rbind(pdf, JTRCIdf [, c("ppid", "change", "RCI", "JTclass_RCI", "time")]) # rbind JTRCIdf with the post data with pdf

# then get the same data for the two follow-ups: 
# 1wk
JTRCI::JTRCI(x.pre= sdf$LSAS.pre, x.post = sdf$LSAS.1wk, reliability = relEstLSAS, ppid = sdf$ppid, plot = F, table = F)
JTRCIdf$time <- "1wk"
pdf <- rbind(pdf, JTRCIdf [, c("ppid", "change", "RCI", "JTclass_RCI", "time")])

# 3mo
JTRCI::JTRCI(x.pre= sdf$LSAS.pre, x.post = sdf$LSAS.3mo, reliability = relEstLSAS, ppid = sdf$ppid, plot = F, table = F)
JTRCIdf$time <- "3mo"
pdf <- rbind(pdf, JTRCIdf [, c("ppid", "change", "RCI", "JTclass_RCI", "time")])

pdf$time <- factor(pdf$time, levels = c ("BL", "post", "1wk", "3mo"))

# merge in group allocation from the BIdf dataframe (in that df we already recoded the original values to G1, G2, G3, and G4)
# Rename group identifiers
BIdf1 <- BIdf
BIdf1$group <- as.character(BIdf1$group)
BIdf1$group[BIdf1$group == "G1"] <- "2D mock"
BIdf1$group[BIdf1$group == "G2"] <- "2D active"
BIdf1$group[BIdf1$group == "G3"] <- "3D mock"
BIdf1$group[BIdf1$group == "G4"] <- "3D active"
BIdf1$group <- factor(BIdf1$group, levels = c("2D mock", "2D active", "3D mock", "3D active"))

pdf <- merge (pdf, BIdf1[, c("ppid", "group")], by = "ppid")

# fix the colors to the classification labels:
levels(pdf$JTclass_RCI)

pdf$JTclass_RCI <- factor(pdf$JTclass_RCI, levels = c("recovered", "non reliably recovered", "improved", "unchanged", "deteriorated"))

# create vector of plot colours linked to specific level names:  
cols <- c("recovered" = "orchid3", "non reliably recovered" = "palevioletred3", "improved" = "olivedrab3", "unchanged" = "skyblue3", "deteriorated" = "coral1")

Sdiff <- unique(JTRCIdf$Sdiff)  # get the SEm value to use for positioning guide lines in the plot. 

# a dataframe to define the lines with - this way we can add a legend for the lines to the plot:  
linesdf =data.frame(s=c(1,2,3), ic=c(0,2,5))
linesdf <- data.frame(cc = c(" 1.96 Sdiff", "   .84 Sdiff", " no change", "   .84 Sdiff", " 1.96 Sdiff"), 
                      ic = c(1.96 * Sdiff, .84 * Sdiff, 0 * Sdiff, -.84 * Sdiff, -1.96 * Sdiff))

levels(linesdf$sd)
linesdf$cc <- factor(linesdf$cc, levels = c (" 1.96 Sdiff", "   .84 Sdiff", " no change"))
##  #if changing anything, notice the added spaces in the factor levels to align the linetype labels (!)

p <- ggplot(data=pdf, aes(x=time, y=change, group = ppid)) +
  geom_line(col = "gray80") + 
  geom_point(aes(col = JTclass_RCI), size = 2.5) + 
  facet_grid(~ group) + 
  scale_y_continuous(limits = c(-65, 25), breaks = seq(-60,20, 10)) +
  geom_abline(data= linesdf, mapping=aes(slope=0, intercept=ic, linetype= factor(cc)), col = "gray50") +
  theme_classic() +
  scale_colour_manual(breaks = c("recovered", "non reliably recovered", "improved", "unchanged", "deteriorated"), values = cols) + 
  scale_linetype_manual(values=c("dashed", "dotted",  "1F")) +
  theme(strip.background = element_blank()) + #remove facet strip background
  labs(x = "time point", y = "LSAS change \n(points relative to baseline score)", colour = "Jacobson-Truax \n classification:", linetype = "line guides: \n") + guides(linetype = guide_legend(order = 2), colour = guide_legend(order = 1, reverse = T))

p

# to export as tiff with various options: remove #
# plotExportTiff(filename = "Fig6 LSAS JT plot.tiff", plot= p, width = 650, height = 350, dpi = 1000)
# to export as tiff with various options: remove #
# plotExportTiff(filename = "Fig6 LSAS JT plot.tiff", 
# plot= p, width = 650, height = 350, dpi = 1000)
# ------------------------------------------------------------------------------

## Secondary outcome measures
## Depression (PHQ-9)
# ------------------------------------------------------------------------------
# Create long data frame for PHQ
PHQlong <- melt(sdf, id.vars = "ppid",
                measure.vars = c("PHQ.pre", "PHQ.1wk", "PHQ.3mo"))


# merge in the group variable from BIdf: 
PHQlong <- merge(PHQlong, BIdf[, c("ppid", "group")], by = "ppid")

# create variables condition (mock/active) & stimuli (2D/3D) based on value of group
PHQlong$condition_string [PHQlong$group %in% c("G1", "G3")] <- "mock"
PHQlong$condition_string [PHQlong$group %in% c("G2", "G4")] <- "active"

PHQlong$condition [PHQlong$group %in% c("G1", "G3")] <- 0
PHQlong$condition [PHQlong$group %in% c("G2", "G4")] <- 1
PHQlong$condition <- as.numeric(PHQlong$condition)

PHQlong$stimuli_string [PHQlong$group %in% c("G1", "G2")] <- "2D"
PHQlong$stimuli_string  [PHQlong$group %in% c("G3", "G4")] <- "3D"

PHQlong$stimuli [PHQlong$group %in% c("G1", "G2")] <- 0
PHQlong$stimuli[PHQlong$group %in% c("G3", "G4")] <- 1
PHQlong$stimuli <- as.numeric(PHQlong$stimuli)

PHQlong$time_string <- sub('.*\\.', '', PHQlong$variable) 
# extract string following the last period in variable "variable" 
# to obtain the timepoint indicators (pre, post, 1wk, 3mo)
PHQlong$time_id [PHQlong$time_string == "pre"] <- 1
PHQlong$time_id [PHQlong$time_string == "post"] <- 2
PHQlong$time_id [PHQlong$time_string == "1wk"] <- 3
PHQlong$time_id [PHQlong$time_string == "3mo"] <- 4

PHQlong$PHQ <- as.numeric(PHQlong$value)

# nlme() mixed effects model on PHQ-9 
PHQ_lme <- lme(PHQ ~ time_id * condition * stimuli,
               random = ~ time_id|ppid,
               data = PHQlong,
               method = 'ML',
               na.action = 'na.omit')
summary(PHQ_lme)
# none of the effects were significant
# ------------------------------------------------------------------------------

## Generalised anxiety (GAD-7)
# ------------------------------------------------------------------------------
# Create long data frame for GAD
GADlong <- melt(sdf, id.vars = "ppid",
                measure.vars = c("GAD.pre", "GAD.1wk", "GAD.3mo"))


# merge in the group variable from BIdf: 
GADlong <- merge(GADlong, BIdf[, c("ppid", "group")], by = "ppid")

# create variables condition (mock/active) & stimuli (2D/3D) based on value of group
GADlong$condition_string [GADlong$group %in% c("G1", "G3")] <- "mock"
GADlong$condition_string [GADlong$group %in% c("G2", "G4")] <- "active"

GADlong$condition [GADlong$group %in% c("G1", "G3")] <- 0
GADlong$condition [GADlong$group %in% c("G2", "G4")] <- 1
GADlong$condition <- as.numeric(GADlong$condition)

GADlong$stimuli_string [GADlong$group %in% c("G1", "G2")] <- "2D"
GADlong$stimuli_string  [GADlong$group %in% c("G3", "G4")] <- "3D"

GADlong$stimuli [GADlong$group %in% c("G1", "G2")] <- 0
GADlong$stimuli[GADlong$group %in% c("G3", "G4")] <- 1
GADlong$stimuli <- as.numeric(GADlong$stimuli)

GADlong$time_string <- sub('.*\\.', '', GADlong$variable) 
# extract string following the last period in variable "variable" 
# to obtain the timepoint indicators (pre, post, 1wk, 3mo)
GADlong$time_id [GADlong$time_string == "pre"] <- 1
GADlong$time_id [GADlong$time_string == "post"] <- 2
GADlong$time_id [GADlong$time_string == "1wk"] <- 3
GADlong$time_id [GADlong$time_string == "3mo"] <- 4

GADlong$GAD <- as.numeric(GADlong$value)

# nlme() mixed effects model on GAD-7 
GAD_lme <- lme(GAD ~ time_id * condition * stimuli,
               random = ~ time_id|ppid,
               data = GADlong,
               method = 'ML',
               na.action = 'na.omit')
summary(GAD_lme)
# none of the effects were significant
# ------------------------------------------------------------------------------

## Emotional regulation (DERS-16)
# ------------------------------------------------------------------------------
# Create long data frame for DERS
DERSlong <- melt(sdf, id.vars = "ppid",
                 measure.vars = c("DERS.pre", "DERS.1wk", "DERS.3mo"))


# merge in the group variable from BIdf: 
DERSlong <- merge(DERSlong, BIdf[, c("ppid", "group")], by = "ppid")

# create variables condition (mock/active) & stimuli (2D/3D) based on value of group
DERSlong$condition_string [DERSlong$group %in% c("G1", "G3")] <- "mock"
DERSlong$condition_string [DERSlong$group %in% c("G2", "G4")] <- "active"

DERSlong$condition [DERSlong$group %in% c("G1", "G3")] <- 0
DERSlong$condition [DERSlong$group %in% c("G2", "G4")] <- 1
DERSlong$condition <- as.numeric(DERSlong$condition)

DERSlong$stimuli_string [DERSlong$group %in% c("G1", "G2")] <- "2D"
DERSlong$stimuli_string  [DERSlong$group %in% c("G3", "G4")] <- "3D"

DERSlong$stimuli [DERSlong$group %in% c("G1", "G2")] <- 0
DERSlong$stimuli[DERSlong$group %in% c("G3", "G4")] <- 1
DERSlong$stimuli <- as.numeric(DERSlong$stimuli)

DERSlong$time_string <- sub('.*\\.', '', DERSlong$variable) 
# extract string following the last period in variable "variable" 
# to obtain the timepoint indicators (pre, post, 1wk, 3mo)
DERSlong$time_id [DERSlong$time_string == "pre"] <- 1
DERSlong$time_id [DERSlong$time_string == "post"] <- 2
DERSlong$time_id [DERSlong$time_string == "1wk"] <- 3
DERSlong$time_id [DERSlong$time_string == "3mo"] <- 4

DERSlong$DERS <- as.numeric(DERSlong$value)

# nlme() mixed effects model on DERS-16 
DERS_lme <- lme(DERS ~ time_id * condition * stimuli,
                random = ~ time_id|ppid,
                data = DERSlong,
                method = 'ML',
                na.action = 'na.omit')
summary(DERS_lme)
# Significant main effect of time, significant time*stimuli interaction
# ------------------------------------------------------------------------------

## Quality of life (BBQ)
# ------------------------------------------------------------------------------
# Create long data frame for BBQ
BBQlong <- melt(sdf, id.vars = "ppid",
                measure.vars = c("BBQ.pre", "BBQ.1wk", "BBQ.3mo"))


# merge in the group variable from BIdf: 
BBQlong <- merge(BBQlong, BIdf[, c("ppid", "group")], by = "ppid")

# create variables condition (mock/active) & stimuli (2D/3D) based on value of group
BBQlong$condition_string [BBQlong$group %in% c("G1", "G3")] <- "mock"
BBQlong$condition_string [BBQlong$group %in% c("G2", "G4")] <- "active"

BBQlong$condition [BBQlong$group %in% c("G1", "G3")] <- 0
BBQlong$condition [BBQlong$group %in% c("G2", "G4")] <- 1
BBQlong$condition <- as.numeric(BBQlong$condition)

BBQlong$stimuli_string [BBQlong$group %in% c("G1", "G2")] <- "2D"
BBQlong$stimuli_string  [BBQlong$group %in% c("G3", "G4")] <- "3D"

BBQlong$stimuli [BBQlong$group %in% c("G1", "G2")] <- 0
BBQlong$stimuli[BBQlong$group %in% c("G3", "G4")] <- 1
BBQlong$stimuli <- as.numeric(BBQlong$stimuli)

BBQlong$time_string <- sub('.*\\.', '', BBQlong$variable) 
# extract string following the last period in variable "variable" 
# to obtain the timepoint indicators (pre, post, 1wk, 3mo)
BBQlong$time_id [BBQlong$time_string == "pre"] <- 1
BBQlong$time_id [BBQlong$time_string == "post"] <- 2
BBQlong$time_id [BBQlong$time_string == "1wk"] <- 3
BBQlong$time_id [BBQlong$time_string == "3mo"] <- 4

BBQlong$BBQ <- as.numeric(BBQlong$value)

# nlme() mixed effects model on BBQ 
ctrl <- lmeControl(opt='optim')

BBQ_lme <- lme(BBQ ~ time_id * condition * stimuli,
               random = ~ time_id|ppid,
               data = BBQlong,
               method = 'ML',
               control = ctrl,
               na.action = 'na.omit')
summary(BBQ_lme)
# No signficant effects
# ------------------------------------------------------------------------------

## Summary of questionnaire scores across time
# ------------------------------------------------------------------------------
t2df <- select(sdf, ppid, Group, LSAS.pre, LSAS.post, LSAS.1wk, LSAS.3mo,
               PHQ.pre, PHQ.1wk, PHQ.3mo,
               GAD.pre, GAD.1wk, GAD.3mo,
               DERS.pre, DERS.1wk, DERS.3mo,
               BBQ.pre, BBQ.1wk, BBQ.3mo)

t2df[, c(3:18)] <- sapply(t2df[, c(3:18)], as.numeric)

t2vars <- c("LSAS.pre", "LSAS.post", "LSAS.1wk", "LSAS.3mo",
            "PHQ.pre", "PHQ.1wk", "PHQ.3mo", 
            "GAD.pre", "GAD.1wk", "GAD.3mo",
            "DERS.pre", "DERS.1wk", "DERS.3mo",
            "BBQ.pre", "BBQ.1wk", "BBQ.3mo")

table2 <- CreateTableOne(vars = t2vars, strata = "Group", data= t2df, 
                         includeNA = FALSE, test = FALSE)

table2
# ------------------------------------------------------------------------------